%
% Convergence experiment with natural color data
% -----------------------------------------------
%
% In this script we reproduce the convergence results for color data.
% We use the color data in the file "data_flowers.mat" under D65
% illumination (see data_color_flowers.m)
%
% We initialize SPCA with sensible parameters for color manifolds [Laparra et al. 12]
% (these parameters could also be obtained by minimizing the projection error,
% see the procedure described in the Technical Report http://isp.uv.es/docs/.
%
%     Initialization takes 25 secs. in a processor of a Dell PowerEdge 2900
%
% Then, we transform 300 test samples and monitorize their convergence by 
% inspecting the evolution of the approximation error using 
% "spca_2_convergencia.m" that returns the approximation error for each
% sample and iteration.
%
%     Transform takes about 50 secs./sample in a processor of a Dell PowerEdge 2900
%
% This program generates the file "convergence_color.mat" in the destination
% folder provided by the user.
%
% These data (together with data coming from exp_converg_synthetic.m and 
% exp_converg_texture.m) have to be analyzed by "convergence_experiments.m" to
% obtain the convergence plots shown in the paper.
%

destination_folder = '/media/disk/vista/Papers/aranyas/general_y_analiticas/general/TNN_2012_reproducible/exp_assumptions/convergence/';

% Raw data (in ATD tristimulus space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('data_flowers','datDatd','paleta_D1_atd')

% Initialization parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


origin=mean(datDatd')';  % Take the origin at the mean of the data 

Nv=4000;                 % Neighbourhood given by 20 % of the samples 

tau=15;                  % Relatively big step size since the manifold is not curved

q=0.06;                  % Relatively big stiffness since the manifold is not curved

criterion=2;             % infomax criterion

Btotal=7;                % Total number of bits to be distributed among the 3 dimensions (global scaling factor for the response space)

Nbins=0;                 % No assumption is made on the bit allocation per dimension (it takes standard bit allocation [Gersho92])

DIST=1;                  % Out of manifold Euclidean distance in ATD space

Aref=0;                  % No assumption is made on the orientation of the axes at the origin (it takes the orientation of the local PCA)

tic
[init_paramD2] =  initialize_SPCA_2(datDatd,origin,Nv,tau,q,criterion,Btotal,Nbins,DIST,Aref);
toc

% Tolerance in the inverse (stopping criterion in the iteration)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tol = 0.001*91;  % 91 is aprox. the maximum euclidean distance in the data set (luminace is in the range [0, 91] cd/m2)
                 % As stated in the paper, in this illustration we allow 0.001*MaxDistance as admisible error in the inverse. 
                 % Note that this is negligible in the ATD space (e.g. 0.1 cd/m2 error in luminance, which is negligible).

% TRANSFORM OF D65 TEST DATA
for nn=1:size(paleta_D1_atd,2),
    tic
    [inv_critD(:,nn) resp_critD(:,nn) inv_euclD(:,nn) resp_euclD(:,nn) EEE(nn).errores] = spca_2_convergencia(datDatd,paleta_D1_atd(:,nn),init_paramD2,tol);
    toc,nn
    save([destination_folder,'convergence_color.mat']) 
end


