%
%  CONVERGENCE EXPERIMENTS
%
%  The ability to reach any point in the manifold using the proposed path 
%  through the geodesic projections (Eq. 15 and Fig. 4 in the paper) is 
%  the key to obtain meaningful transforms with accurate inverse. 
%
%  In these experiments we check the convergence of such procedure in three 
%  cases: 
%     (1) the synthetic 2d curved manifold (spiral data).
%     (2) tristimulus color data from a calibrated color image database 
%         [Laparra et al. 12]
%     (3) spatial texture -luminance only- data from another calibrated 
%          color image database [Olmos04].
%
%  Convergence results in these cases can be reproduced using the specific
%  scripts:
%
%     (1) exp_converg_synthetic.m
%     (2) exp_converg_color.m
%     (3) exp_converg_texture.m
%
%  See these scripts for details. Note that in order to run the summary of
%  results in this script, the above scripts have to be run to get the results
%  in the files (in folders included in the path):
%
%     (1) convergence_synthetic.mat
%     (2) convergence_color.mat
%     (3) convergence_texture.mat
%
%  The current release of the SPCA toolbox includes these files for the
%  user convenience in the folder:
%
%    .../exp_assumptions/convergence/
%
%  This script displays the test data in the input and transformed
%  representations using different SPCA criteria and the convergence
%  curves.
%  
%  Results may slightly differ from those in the paper since test points
%  are randomly chosen.
%


%%%%%%%%%%%%%%%%%%%%%%
% CASO 2D
%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
load convergence_synthetic

lala=[];
for i=1:500,
    lolo=EEE(i).errores;
    if length(lolo)<20;
        lo=[lolo lolo(end)*ones(1,20-length(lolo))];
    end,
    lala=[lala;lo];
end

lala=[];
malos=[];
for i=1:500,
    lolo=EEE(i).errores;
    if (length(lolo)<20)&(length(lolo)>1);
        malos=[malos i];
        lo=[lolo lolo(end)*ones(1,20-length(lolo))];
    end,
    lala=[lala;lo];
end

figure(1),
plot(dat_test(1,:),dat_test(2,:),'b.'),
axis equal,title({'(Synthetic) Spiral data';'Input domain'})

figure(4),
plot(R11(1,:),R11(2,:),'b.'),
axis equal,
axis([-60 30 -10 10])
title({'(Synthetic) Spiral data';'SPCA domain (\gamma=0, plain unfolding)'})

% figure(1),plot(pto_proximo21(1,:),pto_proximo21(2,:),'k.','markersize',8),axis([-11 11 -18 10]),set(gca,'fontsize',17)
% hold on,plot(pto_proximo21(1,malos),pto_proximo21(2,malos),'b.','markersize',19),xlabel('x_1','fontsize',18),ylabel('x_2','fontsize',18)
% print -deps2c convergencia1.eps
% figure(2),plot(R21(1,:),R21(2,:),'k.','markersize',8),axis([-60 30 -10 10]),set(gca,'fontsize',17)
% hold on,plot(R21(1,malos),R21(2,malos),'b.','markersize',19),xlabel('r_1','fontsize',18),ylabel('r_2','fontsize',18)
% print -deps2c convergencia2d_orig.eps

media_syn=mean(lala);
des_syn=std(lala);

% Maximum Euclidean distance

  datos = dat;
  if length(datos(1,:))>1000
     ind=randperm(length(datos(1,:)));
     datos=datos(ind(1:1000));
  end
  n_dat=length(datos(1,:));
  DIST=zeros(n_dat,n_dat);
  for i=1:n_dat
      for j=i:n_dat
          DIST(i,j) = sum((datos(:,i)-datos(:,j)).^2);
      end
  end
  M_syn=sqrt(max(DIST(:)));


%print -deps2c convergencia2d_trans.eps

%%%%%%%%%%%%%%%%%%%%%%
% CASO 3D: florecilla
%%%%%%%%%%%%%%%%%%%%%%


load convergence_color

% Gather reconstruction errors in the inverse as a function of the
% iteration
% 

lala=[];
for i=1:length(EEE),
    lolo=EEE(i).errores;
    if length(lolo)<20;
        lo=[lolo lolo(end)*ones(1,20-length(lolo))];
    else
        lo=lolo; 
    end,
    lala=[lala;lo];
end

% Mean and std of the reconstruction errors in the inverse as a function of the
% iteration

lala=[];
malos=[];
for i=1:length(EEE),
    lolo=EEE(i).errores;
    if (length(lolo)<20)&(length(lolo)>1);
        malos=[malos i];
        lo=[lolo lolo(end)*ones(1,20-length(lolo))];
    end,
    lala=[lala;lo];
end

media_col=mean(lala);
des_col=std(lala);

% Transform data from ATD to LMS color space

load('data_flowers','M_xyz2lms','M_xyz2atd','datDatd')
puntos_D1_lms=M_xyz2lms*inv(M_xyz2atd)*inv_critD;

viu=[-37 23];

figure(2),plot3(puntos_D1_lms(1,:),puntos_D1_lms(2,:),puntos_D1_lms(3,:),'b.','markersize',8),set(gca,'fontsize',20)
axis([0 90 0 70 0 50])
%hold on,plot3(puntos_D1_lms(1,malos),puntos_D1_lms(2,malos),puntos_D1_lms(3,malos),'g.','markersize',19),xlabel('L','fontsize',18),
ylabel('M','fontsize',20)
zlabel('S','fontsize',20)
xlabel('L','fontsize',20)
view([-11 21]),box on
title({'(Real) Color data';'LMS color space'})
%print -deps2c convergencia3d_orig.eps
figure(5),plot3(resp_critD(1,:),resp_critD(2,:),resp_critD(3,:),'b.','markersize',8),axis([-50 50 -7 7 -5 5]),set(gca,'fontsize',20)
%hold on,plot3(resp_critD(1,malos),resp_critD(2,malos),resp_critD(3,malos),'g.','markersize',19),
xlabel('r_1','fontsize',20),ylabel('r_2','fontsize',20),zlabel('r_3','fontsize',20)
view(viu)
axis equal
axis([-45 60 -7 7 -5 5]),box on
title({'(Real) Color data';'SPCA domain (\gamma=0, plain unfolding)'})

%axis equal,
%print -deps2c convergencia3d_trans.eps

% Maximum Euclidean length in the dataset

  datos = datDatd;
  if length(datos(1,:))>1000
     ind=randperm(length(datos(1,:)));
     datos=datos(ind(1:1000));
  end
  n_dat=length(datos(1,:));
  DIST=zeros(n_dat,n_dat);
  for i=1:n_dat
      for j=i:n_dat
          DIST(i,j) = sum((datos(:,i)-datos(:,j)).^2);
      end
  end
  
M_col=sqrt(max(DIST(:)));


%%%%%%%%%%%%%%%%%%%%%%
% CASO 3D: textura
%%%%%%%%%%%%%%%%%%%%%%

load convergence_texture

num=20;

lala=[];
for i=1:length(EEE),
    lolo=EEE(i).errores;
    if length(lolo)<num;
        lo=[lolo lolo(end)*ones(1,num-length(lolo))];
    else
        lo=lolo; 
    end,
    lala=[lala;lo];
end

lala=[];
malos=[];
for i=1:length(EEE),
    lolo=EEE(i).errores;
    if (length(lolo)<num)&(length(lolo)>1);
        malos=[malos i];
        lo=[lolo lolo(end)*ones(1,num-length(lolo))];
    end,
    lala=[lala;lo];
end

media_tex=mean(lala);
des_tex=std(lala);

viu=[-37 23];

figure(3),plot3(pto_proximo21(1,:),pto_proximo21(2,:),pto_proximo21(3,:),'b.','markersize',8),set(gca,'fontsize',20)
%hold on,plot3(pto_proximo21(1,malos),pto_proximo21(2,malos),pto_proximo21(3,malos),'r.','markersize',19),xlabel('L','fontsize',18),
xlabel('C_1','fontsize',20)
ylabel('C_2','fontsize',20)
zlabel('C_3','fontsize',20)
axis([-0.7 0.7 -0.5 0.5 -0.5 0.5])
view([-72 10]),box on,axis equal
title({'(Real) Image texture data';'Samples close to the 3D subspace defined by the DC and two low frequency components'})
%print -deps2c convergencia_textura_orig_1000.eps
% view([90 0]),axis equal
%print -deps2c convergencia_textura_orig_2d_1000_2.eps

figure(6),plot3(R21(1,:),R21(2,:),R21(3,:),'b.','markersize',8),set(gca,'fontsize',20)
%hold on,plot3(R21(1,malos),R21(2,malos),R21(3,malos),'r.','markersize',19),
xlabel('r_1','fontsize',20),ylabel('r_2','fontsize',20),zlabel('r_3','fontsize',20)
axis equal,box on
axis([-100 150 -80 80 -80 80])
view([-72 10])
%print -deps2c convergencia_textura_trans_1000.eps
title({'(Real) Image texture data';'SPCA domain (\gamma=1, infomax)'})
% view([90 0])
%print -deps2c convergencia_textura_trans_2d_1000.eps

  datos = dat;
  if length(datos(1,:))>1000
     ind=randperm(length(datos(1,:)));
     datos=datos(ind(1:1000));
  end
  n_dat=length(datos(1,:));
  DIST=zeros(n_dat,n_dat);
  for i=1:n_dat
      for j=i:n_dat
          DIST(i,j) = sum((datos(:,i)-datos(:,j)).^2);
      end
  end
  M_tex=sqrt(max(DIST(:)));


figure(7),
plot(1:num,media_tex/M_tex,'go-','Markersize',10,'Linewidth',1.5),set(gca,'Fontsize',18)
hold on,plot(1:num,media_col/M_col,'ro-','Markersize',10,'Linewidth',1.5),set(gca,'Fontsize',18)
hold on,plot(1:num,media_syn/M_syn,'bo-','Markersize',10,'Linewidth',1.5),set(gca,'Fontsize',18)
hold on,errorbar(1:num,media_tex/M_tex,(0.5*des_tex/M_tex)/2,'Linewidth',1.5,'color',[0 1 0]),xlabel('iteration','Fontsize',19),ylabel('Error  |x-x_{T}^d|','Fontsize',19)
hold on,errorbar(1:num,media_col/M_col,(0.5*des_col/M_col)/2,'Linewidth',1.5,'color',[1 0 0]),
hold on,errorbar(1:num,media_syn/M_syn,(0.5*des_syn/M_syn)/2,'Linewidth',1.5,'color',[0 0 1]),
legend('(Real) Texture','(Real) Color','(Synthetic) Spiral')
hold on,plot([0 num+1],[0.001 0.001],'k-')
axis([0 num+1 0.0005 0.0025])
title('Convergence of SPCA')
legend boxoff
  
%print -deps2c convergencia_1000_factor.eps

tile